package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.ImmutableMap;
import com.qmth.distributed.print.business.entity.BasicMessage;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.mapper.BasicMessageMapper;
import com.qmth.distributed.print.business.service.BasicMessageService;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.dto.MarkTaskSmsDto;
import com.qmth.teachcloud.common.bean.params.ApproveUserResult;
import com.qmth.teachcloud.common.bean.result.SmsResponseResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysConfig;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.EnumResult;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.MessageEnum;
import com.qmth.teachcloud.common.service.AuthInfoService;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.service.CommonCacheService;
import com.qmth.teachcloud.common.service.SysConfigService;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.common.util.SmsSendUtil;
import com.qmth.teachcloud.mark.service.MarkQuestionService;
import com.qmth.teachcloud.mark.service.MarkUserQuestionService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 短信消息发送
 * @Author: CaoZixuan
 * @Date: 2021-04-28
 */
@Service
public class BasicMessageServiceImpl extends ServiceImpl<BasicMessageMapper, BasicMessage> implements BasicMessageService {
    @Resource
    private SysConfigService sysConfigService;

    @Resource
    private CommonCacheService commonCacheService;

    @Resource
    private SmsSendUtil smsSendUtil;

    @Resource
    private AuthInfoService authInfoService;

    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;

    @Resource
    MarkUserQuestionService markUserQuestionService;
    @Resource
    private MarkQuestionService markQuestionService;

    @Override
    public IPage<BasicMessage> listByMessageType(MessageEnum messageType, Boolean sendStatus, String mobileNumber, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());

        Page<BasicMessage> page = new Page<>(pageNumber, pageSize);
        QueryWrapper<BasicMessage> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicMessage::getSchoolId, schoolId);
        if (!Objects.isNull(messageType)) {
            queryWrapper.lambda().eq(BasicMessage::getMessageType, messageType);
        }
        if (Objects.nonNull(sendStatus)) {
            if (sendStatus) {
                queryWrapper.lambda().eq(BasicMessage::getSendStatus, "OK");
            } else {
                queryWrapper.lambda().ne(BasicMessage::getSendStatus, "OK");
            }
        }
        if (StringUtils.isNotBlank(mobileNumber)) {
            queryWrapper.lambda().like(BasicMessage::getMobileNumber, "%" + mobileNumber + "%");
        }
        if (SystemConstant.longNotNull(dpr.getRequestUserId())) {
            queryWrapper.lambda().eq(BasicMessage::getCreateId, dpr.getRequestUserId());
        }
        if (dpr.getOrgIdSet() != null && dpr.getOrgIdSet().size() > 0) {
            queryWrapper.lambda().in(BasicMessage::getOrgId, dpr.getOrgIdSet());
        }
        queryWrapper.lambda().orderByDesc(BasicMessage::getCreateTime);
        return this.page(page, queryWrapper);
    }

    @Override
    public List<EnumResult> listMessageTypes() {
        return MessageEnum.listTypes();
    }

    @Override
    public void resendSmsTaskById(Long id) {
        BasicMessage basicMessage = this.getById(id);
        if (Objects.isNull(basicMessage)) {
            return;
        }
        this.saveMessageResendLog(basicMessage);
    }

    /**
     * 保存message和备注
     *
     * @param schoolId
     * @param userId
     * @param userName
     * @param mobileNumber
     * @param paperNumber
     * @param courseCode
     * @param variableParams
     * @param messageType
     * @param createId
     * @param remark
     */
    @Override
    public void saveMessageSendLogRemark(Long schoolId, Long userId, String userName, String mobileNumber, String paperNumber, String courseCode, String variableParams, MessageEnum messageType, Long createId, String remark) {
        this.saveMessageRemarkCommon(schoolId, userId, userName, mobileNumber, paperNumber, courseCode, variableParams, messageType, createId, remark);
    }

    /**
     * 短信count
     *
     * @param userId
     * @param mobileNumber
     * @param condition
     * @return
     */
    protected int smsConditionCount(Long userId, String mobileNumber, String condition) {
        QueryWrapper<BasicMessage> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicMessage::getUserId, userId)
                .eq(BasicMessage::getMobileNumber, mobileNumber)
                .eq(BasicMessage::getMessageType, MessageEnum.NOTICE_OF_MARK_TASK_CREATED)
                .eq(BasicMessage::getSendStatus, "OK")
                .and(w -> w.isNotNull(BasicMessage::getRemark).and(x -> x.eq(BasicMessage::getRemark, condition)));
        return this.count(queryWrapper);
    }

    /**
     * 发送阅卷待办短信
     *
     * @param examId
     * @param courseId
     * @param paperNumber
     */
    @Override
    @Transactional
    public void smsMarkTask(Long examId, Long courseId, String paperNumber) {
        List<MarkTaskSmsDto> markTaskSmsDtoList = markUserQuestionService.findMarkTaskSms(examId, courseId, paperNumber);
        if (CollectionUtils.isNotEmpty(markTaskSmsDtoList)) {
            for (MarkTaskSmsDto m : markTaskSmsDtoList) {
                StringJoiner stringJoiner = new StringJoiner("-");
                stringJoiner.add(m.getExamId().toString()).add(m.getCourseId().toString()).add(m.getPaperNumber());
                int count = this.smsConditionCount(m.getUserId(), m.getMobileNumber(), stringJoiner.toString());
                if (count == 0) {
                    this.saveMessageSendLogRemark(m.getSchoolId(), m.getUserId(), m.getRealName(), m.getMobileNumber(), m.getPaperNumber(), null, JSON.toJSONString(ImmutableMap.of("courseName", m.getCourseName(), "examName", m.getExamName())), MessageEnum.NOTICE_OF_MARK_TASK_CREATED, -1L, stringJoiner.toString());
                }
            }
        }
    }

    /**
     * 发送短信
     *
     * @param userId
     * @param userName
     * @param mobileNumber
     * @param paperNumber
     * @param courseCode
     * @param variableParams
     * @param messageType
     * @param createId
     * @param remark
     */
    public void saveMessageSendLog(Long schoolId, Long userId, String userName, String mobileNumber, String paperNumber, String courseCode, String variableParams, MessageEnum messageType, Long createId, String remark) {
        if (!authInfoService.allowPhoneLogin(schoolId)) {
            return;
        }
        this.saveMessageRemarkCommon(schoolId, userId, userName, mobileNumber, paperNumber, courseCode, variableParams, messageType, createId, remark);
    }

    /**
     * 保存消息和备注
     *
     * @param schoolId
     * @param userId
     * @param userName
     * @param mobileNumber
     * @param paperNumber
     * @param courseCode
     * @param variableParams
     * @param messageType
     * @param createId
     * @param remark
     */
    @Transactional
    public void saveMessageRemarkCommon(Long schoolId, Long userId, String userName, String mobileNumber, String paperNumber, String courseCode, String variableParams, MessageEnum messageType, Long createId, String remark) {
        BasicMessage basicMessage = new BasicMessage();
        String templateCode = null;
        try {
            // code和content
            Map<String, String> enumInfo = smsSendUtil.getCodeAndContentByEnum(messageType);
            if (!enumInfo.containsKey("templateCode")) {
                throw ExceptionResultEnum.ERROR.exception("未找到短信模板Code");
            }
            templateCode = enumInfo.get("templateCode");
            if (StringUtils.isBlank(templateCode)) {
                throw ExceptionResultEnum.ERROR.exception("阿里云短信模板Code必填");
            }

            // 发送短信参数配置验证
            SysConfig sysConfig = sysConfigService.getByKey("sys.message.enable");
            if (sysConfig == null) {
                throw ExceptionResultEnum.ERROR.exception("未找到发送短信开关配置参数，默认不发送");
            }
            if (sysConfig.getConfigValue() == null) {
                throw ExceptionResultEnum.ERROR.exception("未正确配置发送短信开关参数，默认不发送");
            }
            if (sysConfig.getConfigValue().equals("false")) {
                throw ExceptionResultEnum.ERROR.exception("发送短信开关设置为已关闭");
            }

            // 参数校验
            if (StringUtils.isBlank(userName)) {
                throw ExceptionResultEnum.ERROR.exception("用户名称必填");
            }
            if (StringUtils.isBlank(mobileNumber)) {
                throw ExceptionResultEnum.ERROR.exception("用户手机号必填");
            }
            if (StringUtils.isBlank(variableParams)) {
                throw ExceptionResultEnum.ERROR.exception("短信内容参数值必填");
            }

            // 调用阿里云短信平台发送短信
            Map<String, Object> templateParam = JSON.parseObject(variableParams, Map.class);
            SmsResponseResult smsResponseResult = smsSendUtil.sendSms(mobileNumber, templateCode, templateParam);
            if (!SmsSendUtil.OK.equals(smsResponseResult.getCode())) {
                throw ExceptionResultEnum.ERROR.exception("阿里云短信发送接口调用失败");
            }
            basicMessage.setSendStatus(smsResponseResult.getCode());
            basicMessage.setSendResult(smsResponseResult.getMessage());
        } catch (Exception e) {
            basicMessage.setSendStatus("SYSTEM_ERROR");
            basicMessage.setSendResult(e.getMessage());
        } finally {
            basicMessage.setId(SystemConstant.getDbUuid());
            basicMessage.setSchoolId(schoolId);
            basicMessage.setUserId(userId);
            basicMessage.setUserName(userName);
            basicMessage.setMobileNumber(mobileNumber);
            basicMessage.setPaperNumber(paperNumber);
            basicMessage.setCourseCode(courseCode);
            basicMessage.setTemplateCode(templateCode);
            basicMessage.setVariableParams(variableParams);
            basicMessage.setMessageType(messageType);
            basicMessage.setBusinessOperate(messageType.getName());
            basicMessage.setRemark(remark);
            basicMessage.setResendCount(0);
            basicMessage.setCreateId(createId);
            basicMessage.setCreateTime(System.currentTimeMillis());
            this.save(basicMessage);
        }
    }

    @Override
    public void saveMessageSendLog(Long schoolId, Long userId, String userName, String mobileNumber, String variableParams, MessageEnum messageType) {
        BasicMessage basicMessage = new BasicMessage();
        String templateCode = null;
        try {
            // code和content
            Map<String, String> enumInfo = smsSendUtil.getCodeAndContentByEnum(messageType);
            if (!enumInfo.containsKey("templateCode")) {
                throw ExceptionResultEnum.ERROR.exception("未找到短信模板Code");
            }
            templateCode = enumInfo.get("templateCode");
            if (StringUtils.isBlank(templateCode)) {
                throw ExceptionResultEnum.ERROR.exception("阿里云短信模板Code必填");
            }

            if (StringUtils.isBlank(mobileNumber)) {
                throw ExceptionResultEnum.ERROR.exception("用户手机号必填");
            }
            if (StringUtils.isBlank(variableParams)) {
                throw ExceptionResultEnum.ERROR.exception("短信内容参数值必填");
            }

            // 调用阿里云短信平台发送短信
            Map<String, Object> templateParam = JSON.parseObject(variableParams, Map.class);
            SmsResponseResult smsResponseResult = smsSendUtil.sendSms(schoolId, mobileNumber, templateCode, templateParam);
            if (!SmsSendUtil.OK.equals(smsResponseResult.getCode())) {
                throw ExceptionResultEnum.ERROR.exception("阿里云短信发送接口调用失败");
            }
            basicMessage.setSendStatus(smsResponseResult.getCode());
            basicMessage.setSendResult(smsResponseResult.getMessage());

        } catch (Exception e) {
            basicMessage.setSendStatus("SYSTEM_ERROR");
            basicMessage.setSendResult(e.getMessage());
        } finally {
            basicMessage.setId(SystemConstant.getDbUuid());
            basicMessage.setSchoolId(schoolId);
            basicMessage.setUserId(userId);
            basicMessage.setUserName(userName);
            basicMessage.setMobileNumber(mobileNumber);
            basicMessage.setTemplateCode(templateCode);
            basicMessage.setVariableParams(variableParams);
            basicMessage.setMessageType(messageType);
            basicMessage.setBusinessOperate(messageType.getName());
            basicMessage.setResendCount(0);
            basicMessage.setCreateTime(System.currentTimeMillis());
            this.save(basicMessage);
        }
    }

    /**
     * 失败重发
     *
     * @param basicMessage
     */
    public void saveMessageResendLog(BasicMessage basicMessage) {
        try {
            if (basicMessage == null) {
                return;
            }
            if (!authInfoService.allowPhoneLogin(basicMessage.getSchoolId())) {
                return;
            }
            if ("OK".equals(basicMessage.getSendStatus())) {
                return;
            }

            if (!StringUtils.isBlank(basicMessage.getMobileNumber())) {
                // 调用阿里云短信平台发送短信
                Map<String, Object> templateParam = JSON.parseObject(basicMessage.getVariableParams(), Map.class);
                SmsResponseResult smsResponseResult = smsSendUtil.sendSms(basicMessage.getMobileNumber(), basicMessage.getTemplateCode(), templateParam);
                if (!SmsSendUtil.OK.equals(smsResponseResult.getCode())) {
                    throw ExceptionResultEnum.ERROR.exception("阿里云短信发送接口调用失败");
                }
                basicMessage.setSendStatus(smsResponseResult.getCode());
                basicMessage.setSendResult(smsResponseResult.getMessage());
            }
        } catch (Exception e) {
            basicMessage.setSendStatus("SYSTEM_ERROR");
            basicMessage.setSendResult(e.getMessage());
        } finally {
            // 重试次数+1
            basicMessage.setResendCount(basicMessage.getResendCount() + 1);
            this.updateById(basicMessage);
        }
    }


    /**
     * 发送给命题老师-待办提醒
     *
     * @param examTasks
     */
    @Override
    public void sendNoticeTaskCreate(ExamTask... examTasks) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        // 指派命题老师的命题任务
        Map<Long, List<String>> collects = Arrays.stream(examTasks).filter(m -> m.getUserId() != null).collect(Collectors.groupingBy(ExamTask::getUserId, Collectors.mapping(m -> String.valueOf(m.getId()), Collectors.toList())));
        for (Map.Entry<Long, List<String>> longListEntry : collects.entrySet()) {
            Long userId = longListEntry.getKey();
            List<String> ids = longListEntry.getValue();
            SysUser user = commonCacheService.userCache(userId);
            if (Objects.nonNull(user)) {
                String userName = user.getRealName();
                String mobileNumber = user.getMobileNumber();

                Map<String, Object> jsonMap = new HashMap<>();
                jsonMap.put("userName", userName);
                jsonMap.put("count", ids.size());
                String variableParams = JSON.toJSONString(jsonMap);
                String remark = String.format("命题任务ID：%s", String.join(",", ids));
                this.saveMessageSendLog(schoolId, userId, userName, mobileNumber, null, null, variableParams, MessageEnum.NOTICE_OF_EXAM_TASK_CREATED, sysUser.getId(), remark);
            } else {
                String remark = String.format("命题任务ID：%s，用户不存在", String.join(",", ids));
                this.saveMessageSendLog(schoolId, userId, null, null, null, null, null, MessageEnum.NOTICE_OF_EXAM_TASK_CREATED, sysUser.getId(), remark);
            }
        }
    }

    @Override
    public void sendNoticeTaskAuditCreateOrReview(ExamTask examTask, MessageEnum messageType) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        // 考务老师(命题任务创建人)
        Long userId = examTask.getCreateId();
        SysUser user = commonCacheService.userCache(userId);
        if (Objects.nonNull(user)) {
            String userName = user.getRealName();
            String mobileNumber = user.getMobileNumber();
            String paperNumber = examTask.getPaperNumber();
            String courseCode = examTask.getCourseCode();
            String courseName = examTask.getCourseName();

            Map<String, Object> jsonMap = new HashMap<>();
            jsonMap.put("userName", userName);
            StringJoiner sj = new StringJoiner(SystemConstant.PAUSE_SIGN).add(courseName).add(paperNumber);
            jsonMap.put("courseName", sj.toString());
            String variableParams = JSON.toJSONString(jsonMap);
            String remark = String.format("命题任务ID：%s", examTask.getId().toString());
            this.saveMessageSendLog(schoolId, userId, userName, mobileNumber, paperNumber, courseCode, variableParams, messageType, sysUser.getId(), remark);
        } else {
            String remark = String.format("命题任务ID：%s，用户不存在", examTask.getId().toString());
            this.saveMessageSendLog(schoolId, userId, null, null, null, null, null, messageType, sysUser.getId(), remark);
        }
    }

    @Override
    public void sendNoticeExpireOrOverdue(Long schoolId, MessageEnum messageType, Long userId, List<String> ids) {
        // 考务老师(命题任务创建人)
        SysUser user = commonCacheService.userCache(userId);
        if (Objects.nonNull(user)) {
            String userName = user.getRealName();
            String mobileNumber = user.getMobileNumber();
            Map<String, Object> jsonMap = new HashMap<>();
            jsonMap.put("userName", userName);
            jsonMap.put("count", ids.size());
            String variableParams = JSON.toJSONString(jsonMap);
            String remark = String.format("命题任务ID：%s", String.join(",", ids));
            this.saveMessageSendLog(schoolId, userId, userName, mobileNumber, null, null, variableParams, messageType, null, remark);
        } else {
            String remark = String.format("命题任务ID：%s，用户不存在", String.join(",", ids));
            this.saveMessageSendLog(schoolId, userId, null, null, null, null, null, messageType, null, remark);
        }
    }

    @Override
    public void resendSmsTask() {
        /*SysConfig sysConfig = sysConfigService.getByKey("sys.message.resendCount");
        if (sysConfig != null) {
            Integer resendCount = StringUtils.isBlank(sysConfig.getConfigValue()) ? 0 : Integer.valueOf(sysConfig.getConfigValue());
            QueryWrapper<BasicMessage> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().ne(BasicMessage::getSendStatus, "OK").lt(BasicMessage::getResendCount, resendCount);
            List<BasicMessage> basicMessageList = this.list(queryWrapper);
            if (basicMessageList != null && basicMessageList.size() > 0) {
                for (BasicMessage basicMessage : basicMessageList) {
                    this.saveMessageResendLog(basicMessage);
                }
            }
        }*/
    }

    @Override
    public void sendNoticeTaskAuditFlow(ExamTask examTask, List<ApproveUserResult> users, MessageEnum messageType) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        // 考务老师(命题任务创建人)
        for (ApproveUserResult user : users) {
            String userName = user.getRealName();
            String mobileNumber = user.getMobileNumber();
            String paperNumber = examTask.getPaperNumber();
            String courseCode = examTask.getCourseCode();
            String courseName = examTask.getCourseName();
            Map<String, Object> jsonMap = new HashMap<>();
            jsonMap.put("userName", userName);
            StringJoiner sj = new StringJoiner(SystemConstant.PAUSE_SIGN).add(courseName).add(paperNumber);
            jsonMap.put("courseName", sj.toString());
            String variableParams = JSON.toJSONString(jsonMap);
            String remark = String.format("命题任务ID：%s", examTask.getId().toString());
            this.saveMessageSendLog(schoolId, user.getId(), userName, mobileNumber, paperNumber, courseCode, variableParams, messageType, sysUser.getId(), remark);
        }
    }

    /**
     * 发送审核抄送短信
     *
     * @param examTask
     * @param users
     * @param auditUserNames
     * @param messageType
     */
    @Override
    public void sendNoticeAuditCopyUser(ExamTask examTask, List<ApproveUserResult> users, String auditUserNames, MessageEnum messageType) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        for (ApproveUserResult user : users) {
            String userName = user.getRealName();
            String mobileNumber = user.getMobileNumber();
            String paperNumber = examTask.getPaperNumber();
            String courseCode = examTask.getCourseCode();
            String courseName = examTask.getCourseName();
            Map<String, Object> jsonMap = new HashMap<>();
            jsonMap.put("userName", userName);
            StringJoiner sj = new StringJoiner(SystemConstant.PAUSE_SIGN).add(courseName).add(paperNumber);
            jsonMap.put("courseName", sj.toString());
            jsonMap.put("auditUserNames", auditUserNames);
            String variableParams = JSON.toJSONString(jsonMap);
            String remark = String.format("命题任务ID：%s", examTask.getId().toString());
            this.saveMessageSendLog(schoolId, user.getId(), userName, mobileNumber, paperNumber, courseCode, variableParams, messageType, sysUser.getId(), remark);
        }
    }
}
