package com.qmth.distributed.print.api.admin;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.params.SysAdminSetParam;
import com.qmth.distributed.print.business.service.BasicExamService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicExam;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.data.entity.TSyncData;
import com.qmth.teachcloud.data.service.DataService;
import com.qmth.teachcloud.data.service.TSyncDataService;
import com.qmth.teachcloud.task.enums.JobEnum;
import com.qmth.teachcloud.task.job.DataSyncJob;
import com.qmth.teachcloud.task.service.QuartzService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.Scope;
import org.apache.commons.lang3.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * 客户端升级 前端控制器
 */
@Api(tags = "超管-数据同步Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_SYS + "/school/data")
@Aac(auth = false)
public class DataController {

    @Resource
    private BasicExamService basicExamService;
    @Resource
    private QuartzService quartzService;
    @Resource
    private DataService dataService;
    @Resource
    private TSyncDataService tSyncDataService;

    @ApiOperation(value = "测试连接")
    @RequestMapping(value = "/test/connect", method = RequestMethod.POST)
    public Result testConnect(@Valid @RequestBody SysAdminSetParam sysAdminSetParam, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        Optional.ofNullable(sysAdminSetParam.getParam()).orElseThrow(() -> ExceptionResultEnum.PARAMS_ERROR.exception("数据同步参数未填写"));
        dataService.testConnect(sysAdminSetParam);
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "查看设置列表")
    @RequestMapping(value = "/param/list", method = RequestMethod.POST)
    public Result listParam(@ApiParam(value = "学校ID") @RequestParam(value = "schoolId") Long schoolId,
                            @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                            @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(tSyncDataService.listParam(schoolId, pageNumber, pageSize));
    }

    @ApiOperation(value = "新增/修改同步任务")
    @RequestMapping(value = "/param/save", method = RequestMethod.POST)
    public Result paramSave(@ApiParam(value = "任务对象") @RequestBody TSyncData tSyncData) {
        TSyncData saveData = tSyncDataService.saveData(tSyncData);
        if (saveData != null) {
            quartzTask(saveData);
        }
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "手动同步")
    @RequestMapping(value = "/start/sync", method = RequestMethod.POST)
    public Result startSync(@ApiParam(value = "学校ID") @RequestParam(value = "schoolId") Long schoolId,
                            @ApiParam(value = "考试ID") @RequestParam(value = "examId") Long examId) {
        TSyncData tSyncData = tSyncDataService.selectByMultiId(new TSyncData(schoolId, examId));
        if (tSyncData == null) {
            throw ExceptionResultEnum.ERROR.exception("同步任务不存在");
        } else if (tSyncData.getStatus()) {
            throw ExceptionResultEnum.ERROR.exception("正在同步中，请稍后再试");
        }
        quartzService.runAJobNow(JobEnum.DATA_SYNC.name() + "-" + schoolId + "-" + examId, JobEnum.DATA_SYNC.getGroupName());
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "查看日志")
    @RequestMapping(value = "/sync/log", method = RequestMethod.POST)
    public Result listLog(@ApiParam(value = "学校ID") @RequestParam(value = "schoolId") Long schoolId,
                          @ApiParam(value = "考试ID") @RequestParam(value = "examId") Long examId,
                          @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                          @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(tSyncDataService.getDataLog(schoolId, examId, pageNumber, pageSize));
    }

    @ApiOperation(value = "启用/禁用")
    @RequestMapping(value = "/enable/sync", method = RequestMethod.POST)
    public Result enable(@RequestBody TSyncData tSyncData) {
        if (tSyncDataService.enable(tSyncData)) {
            quartzTask(tSyncDataService.selectByMultiId(tSyncData));
        }
        return ResultUtil.ok(true);
    }


    private void quartzTask(TSyncData syncData) {
        Long startTime = syncData.getStartTime();
        Long endTime = syncData.getEndTime();
        BasicExam basicExam = basicExamService.getById(syncData.getExamId());
        // 考试已禁用、任务已禁用、未到开始时间、超过结束时间，未设置执行时间 。不执行
        quartzService.deleteJob(JobEnum.DATA_SYNC.name() + "-" + syncData.getSchoolId() + "-" + syncData.getExamId(), JobEnum.DATA_SYNC.getGroupName());
        long currentTime = System.currentTimeMillis();
        if (!basicExam.getEnable() || !syncData.getEnable() || currentTime < startTime || currentTime > endTime || StringUtils.isBlank(syncData.getCron())) {
            return;
        }
        Map expireJobMap = new HashMap();
        expireJobMap.computeIfAbsent("name", v -> DataSyncJob.class.getName());
        quartzService.addJob(DataSyncJob.class, JobEnum.DATA_SYNC.name() + "-" + syncData.getSchoolId() + "-" + syncData.getExamId(), JobEnum.DATA_SYNC.getGroupName(), syncData.getCron(), expireJobMap);
    }
}

