package com.qmth.distributed.print.api.mark;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.mark.MarkMode;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.dto.mark.mark.SubmitResult;
import com.qmth.teachcloud.mark.entity.MarkUserQuestion;
import com.qmth.teachcloud.mark.enums.LockType;
import com.qmth.teachcloud.mark.enums.QuestionModel;
import com.qmth.teachcloud.mark.lock.LockService;
import com.qmth.teachcloud.mark.params.MarkResult;
import com.qmth.teachcloud.mark.params.MarkResultQuestion;
import com.qmth.teachcloud.mark.service.MarkService;
import com.qmth.teachcloud.mark.service.MarkUserPaperService;
import com.qmth.teachcloud.mark.service.MarkUserQuestionService;
import com.qmth.teachcloud.mark.utils.TaskLock;
import com.qmth.teachcloud.mark.utils.TaskLockUtil;
import io.lettuce.core.GeoArgs.Sort;
import io.swagger.annotations.Api;
import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 评卷端接口 前端控制器
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Api(tags = "评卷-评卷接口")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/mark")
public class MarkController extends BaseController {
    private final static Logger log = LoggerFactory.getLogger(MarkController.class);

    @Autowired
    private MarkService markService;
    @Resource
    private MarkUserPaperService markUserPaperService;
    @Autowired
    private SysUserService userService;
    @Resource
    private MarkUserQuestionService markUserQuestionService;
    @Resource
    LockService lockService;

    /**
     * 查询内存数据
     *
     * @param examId      考试ID
     * @param paperNumber 试卷编号
     */
    @PostMapping(value = "/")
    @Aac(auth = false)
    public Object getTaskLock(@RequestParam(required = false) Long examId,
                              @RequestParam(required = false) String paperNumber) {
        Map<String, TaskLock> formalTaskMap = TaskLockUtil.getFormalTaskMap();
        if (examId != null && StringUtils.isNotBlank(paperNumber)) {
            TaskLock taskLock = TaskLockUtil.getFormalTask(examId + "_" + paperNumber);
            return taskLock.list();
        }
        Map<String, List<Map<String, Object>>> map = new HashMap<>();
        for (Map.Entry<String, TaskLock> entry : formalTaskMap.entrySet()) {
            map.put(entry.getKey(), entry.getValue().list());
        }
        return map;
    }

    @RequestMapping(value = "/getSetting", method = RequestMethod.POST)
    public Result getSetting(@RequestParam Long examId,
                             @RequestParam String paperNumber) {
        SysUser user = (SysUser) ServletUtil.getRequestUser();
        return ResultUtil.ok(markService.getSetting(user, examId, paperNumber));
    }

    @RequestMapping(value = "/getStatus", method = RequestMethod.POST)
    public Result status(@RequestParam Long examId, @RequestParam String paperNumber,
                         @RequestParam QuestionModel questionModel,
                         @RequestParam(required = false) Long questionId) {
        validMarkPaperForMark(examId, paperNumber);
        SysUser user = (SysUser) ServletUtil.getRequestUser();
        return ResultUtil.ok(markService.getStatus(user.getId(), examId, paperNumber, questionModel, questionId));
    }

    @RequestMapping(value = "/getTask", method = RequestMethod.POST)
    public Result getTask(@RequestParam Long examId,
                          @RequestParam String paperNumber,
                          @RequestParam QuestionModel questionModel,
                          @RequestParam(required = false) Long questionId) {
        SysUser user = (SysUser) ServletUtil.getRequestUser();
        return ResultUtil.ok(markService.getTask(user.getId(), examId, paperNumber, questionModel, questionId));
    }

    @RequestMapping(value = "/saveTask", method = RequestMethod.POST)
    public Result saveTask(@RequestParam Long examId, @RequestParam String paperNumber,
                           @RequestBody MarkResult result) {
        validMarkPaperForMark(examId, paperNumber);
        SysUser user = (SysUser) ServletUtil.getRequestUser();
        Long userId = user.getId();

        for (MarkResultQuestion question : result.getQuestionList()) {
            MarkUserQuestion markUserQuestion = markUserQuestionService.getByExamIdAndPaperNumberAndQuestionIdAndUserId(examId, paperNumber, question.getQuestionId(), userId);
            if (markUserQuestion == null) {
                throw ExceptionResultEnum.ERROR.exception("评卷员未绑定题目" + question.getMainNumber() + "-" + question.getSubNumber());
            }
        }

        SubmitResult sr;
        try {
            lockService.watch(LockType.EXAM_SUBJECT, examId, paperNumber);
            lockService.watch(LockType.MARK_USER_QUESTION, userId);
            lockService.waitlock(LockType.STUDENT, result.getStudentId());
            sr = markService.saveTask(examId, paperNumber, user.getId(), result);
            for (MarkResultQuestion markResultQuestion : result.getQuestionList()) {
                markService.updateMarkedCount(examId, paperNumber, markResultQuestion.getQuestionId());
            }
        } catch (Exception e) {
            log.error("save task error", e);
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        } finally {
            lockService.unlock(LockType.STUDENT, result.getStudentId());
            lockService.unwatch(LockType.MARK_USER_QUESTION, userId);
            lockService.unwatch(LockType.EXAM_SUBJECT, examId, paperNumber);
        }

        return ResultUtil.ok(sr);
    }

    @RequestMapping(value = "/getHistory", method = RequestMethod.POST)
    public Result getHistory(@RequestParam int pageNumber, @RequestParam int pageSize, @RequestParam String order, @RequestParam String sort,
                             @RequestParam Long examId, @RequestParam String paperNumber, @RequestParam(required = false) String secretNumber, @RequestParam(required = false) Double markerScore) {
        SysUser user = (SysUser) ServletUtil.getRequestUser();
        return ResultUtil.ok(markService.getHistory(user.getId(), pageNumber, pageSize, "DESC".equalsIgnoreCase(sort) ? Sort.desc : Sort.asc, order, examId, paperNumber, secretNumber, markerScore));
    }

    @RequestMapping(value = "/updateSetting", method = RequestMethod.POST)
    public Result updateSetting(@RequestParam Long examId,
                                @RequestParam String paperNumber,
                                @RequestParam String uiSetting,
                                @RequestParam MarkMode mode,
                                @RequestParam(required = false) QuestionModel questionModel) {
        SysUser user = (SysUser) ServletUtil.getRequestUser();
        userService.updateUiSetting(user.getId(), StringEscapeUtils.unescapeHtml(StringUtils.trimToNull(uiSetting)));
        markUserPaperService.update(examId, paperNumber, user.getId(), mode, questionModel, null);
        return ResultUtil.ok(true);
    }

    @RequestMapping(value = "/clear", method = RequestMethod.POST)
    public Result clear(@RequestParam Long examId, @RequestParam String paperNumber) {
        SysUser user = (SysUser) ServletUtil.getRequestUser();
        markService.clear(user.getId(), examId, paperNumber);
        return ResultUtil.ok(true);
    }
}
