package com.qmth.distributed.print.upgrade;

import com.alibaba.fastjson.JSON;
import com.qmth.boot.data.upgrade.annotation.DataUpgradeVersion;
import com.qmth.boot.data.upgrade.service.DataUpgradeService;
import com.qmth.boot.data.upgrade.utils.ResourceFileHelper;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.entity.ExamTaskDetail;
import com.qmth.teachcloud.common.bean.vo.PaperInfoVo;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.mark.entity.MarkQuestionAnswer;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.BatchPreparedStatementSetter;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.*;

/**
 * 升级3.4.2后，3.4.1及以前历史数据的处理
 */
@DataUpgradeVersion("3.4.2")
public class DataUpgrade_3_4_2 implements DataUpgradeService {

    private static final Logger log = LoggerFactory.getLogger(DataUpgrade_3_4_2.class);

    // 一页1000条
    private static int PAGE_SIZE = 500;

    private static String DEFAULT_A = "A";

    private static String DATE_PATTERN = "yyyy-MM-dd HH:mm:ss";

    @Override
    public void process(JdbcTemplate jdbcTemplate) {
        log.info("process...");
        // 执行历史数据归档
        this.execSerialNumber(jdbcTemplate);
    }

    private void execSerialNumber(JdbcTemplate jdbcTemplate) {
        List<ExamTaskDetail> examTaskDetailSaveList = new ArrayList<>();
        List<ExamTaskDetail> examTaskDetailUpdateList = new ArrayList<>();

        log.info("开始处理历史数据；时间：" + DateFormatUtils.format(new Date(), DATE_PATTERN));
        List<ExamTaskDetail> examTaskDetails = this.examTaskDetailList(jdbcTemplate);
        log.info("1.1开始拆分电子交卷上传试卷数据，共" + examTaskDetails.size() + "条数据");
        // 拆分电子交卷试卷
        for (ExamTaskDetail examTaskDetail : examTaskDetails) {
            ExamTask examTask = this.getExamTaskById(jdbcTemplate, examTaskDetail.getExamTaskId());
            if (examTask == null) {
                continue;
            }

            String paperType = examTaskDetail.getPaperType();
            // 所有卷型
            String[] paperTypes = paperType.split(",");
            // 已曝光集合
            String exposedPaperType = examTaskDetail.getExposedPaperType();
            List<String> exposePaperTypes = StringUtils.isNotBlank(exposedPaperType) ? Arrays.asList(exposedPaperType.split(",")) : Collections.emptyList();
            // 已关联集合
            String relatePaperType = examTaskDetail.getRelatePaperType();
            List<String> relatePaperTypes = StringUtils.isNotBlank(relatePaperType) ? Arrays.asList(relatePaperType.split(",")) : Collections.emptyList();
            // 所有上传卷型试卷信息
            List<PaperInfoVo> paperInfoVoList = examTaskDetail.getPaperInfoVoList();
            // 只有一个卷型、或者开启AB卷的，直接更新
            if (paperType.length() == 1 || examTask.getOpenAb()) {
                examTaskDetail.setSerialNumber(1);
                examTaskDetail.setExposed(exposePaperTypes.contains(paperType));
                examTaskDetail.setExposedExamId(examTaskDetail.getExposed() || relatePaperTypes.contains(paperType) ? examTask.getExamId() : null);
                examTaskDetailUpdateList.add(examTaskDetail);
            } else {
                for (String type : paperTypes) {
                    // 暂时不处理A
                    if (type.equals(DEFAULT_A)) {
                        continue;
                    }
                    ExamTaskDetail taskDetail = new ExamTaskDetail();
                    taskDetail.setExamTaskId(examTaskDetail.getExamTaskId());
                    taskDetail.setSerialNumber(calcSerialNumber(type));
                    taskDetail.setPaperType(type);
                    taskDetail.setPaperAttachmentIds(getPaperTypeIds(paperInfoVoList, type));
                    taskDetail.setExposed(exposePaperTypes.contains(type));
                    taskDetail.setExposedExamId(taskDetail.getExposed() || relatePaperTypes.contains(type) ? examTask.getExamId() : null);
                    taskDetail.setCreateId(examTaskDetail.getCreateId());
                    taskDetail.setCreateTime(examTaskDetail.getCreateTime());
                    taskDetail.setUpdateId(examTaskDetail.getUpdateId());
                    taskDetail.setUpdateTime(examTaskDetail.getUpdateTime());
                    examTaskDetailSaveList.add(taskDetail);

                }

                // 最后处理A卷，将原数据改为A
                examTaskDetail.setSerialNumber(1);
                examTaskDetail.setPaperType(DEFAULT_A);
                examTaskDetail.setPaperAttachmentIds(getPaperTypeIds(paperInfoVoList, DEFAULT_A));
                examTaskDetail.setExposed(exposePaperTypes.contains(DEFAULT_A));
                examTaskDetail.setExposedExamId(examTaskDetail.getExposed() || relatePaperTypes.contains(DEFAULT_A) ? examTask.getExamId() : null);
                examTaskDetailUpdateList.add(examTaskDetail);
            }
        }
        this.saveBatchExamTaskDetail(jdbcTemplate, examTaskDetailSaveList);
        this.updateBatchExamTaskDetail(jdbcTemplate, examTaskDetailUpdateList);
        log.info("1.1线束拆分电子交卷上传试卷数据，共" + examTaskDetails.size() + "条数据");

        log.info("1.2开始更新exam_detail_course表中serial_number值");
        // 更新exam_detail_course表serial_number
        this.updateExamDetailCourseSerialNumber(jdbcTemplate);
        log.info("1.2结束更新exam_detail_course表中serial_number值");

        log.info("1.3开始更新mark_paper表中serial_number值");
        // 更新mark_paper表serial_number
        this.updateMarkPaperSerialNumber(jdbcTemplate);
        log.info("1.3结束更新mark_paper表中serial_number值");

        log.info("1.4开始更新mark_paper_package表中serial_number值");
        // 更新mark_paper_package表serial_number
        this.updateMarkPaperPackageSerialNumber(jdbcTemplate);
        log.info("1.4结束更新mark_paper_package表中serial_number值");

        log.info("1.5开始更新mark_student表中serial_number值");
        // 更新mark_student表serial_number
        this.updateMarkStudentSerialNumber(jdbcTemplate);
        log.info("1.5结束更新mark_student表中serial_number值");

        log.info("1.6开始更新scan_answer_card表中serial_number值");
        // 更新scan_answer_card表serial_number
        this.updateScanAnswerCardSerialNumber(jdbcTemplate);
        log.info("1.6结束更新scan_answer_card表中serial_number值");

        log.info("2.1开始拆分mark_question表中客观题标答到mark_question_answer表");
        // 拆分客观题标答
        List<MarkQuestionAnswer> markQuestionAnswerList = new ArrayList<>();
        for (MarkQuestion markQuestion : this.markQuestionList(jdbcTemplate)) {
            MarkQuestionAnswer markQuestionAnswer = new MarkQuestionAnswer();
            markQuestionAnswer.setExamId(markQuestion.getExamId());
            markQuestionAnswer.setPaperNumber(markQuestion.getPaperNumber());
            markQuestionAnswer.setPaperType(markQuestion.getPaperType());
            markQuestionAnswer.setMainNumber(markQuestion.getMainNumber());
            markQuestionAnswer.setSubNumber(markQuestion.getSubNumber());
            markQuestionAnswer.setAnswer(markQuestion.getAnswer());
            markQuestionAnswer.setObjectivePolicy(markQuestion.getObjectivePolicy());
            markQuestionAnswer.setObjectivePolicyScore(markQuestion.getObjectivePolicyScore());
            markQuestionAnswerList.add(markQuestionAnswer);
        }
        if (CollectionUtils.isNotEmpty(markQuestionAnswerList)) {
            this.saveBatchMarkQuestionAnswer(jdbcTemplate, markQuestionAnswerList);
        }
        log.info("2.1结束拆分mark_question表中客观题标答到mark_question_answer表");
        log.info("历史数据处理完成；时间：" + DateFormatUtils.format(new Date(), DATE_PATTERN));

    }

    private void execUpgradeSql(JdbcTemplate jdbcTemplate, String upgradeSql) {
        if (StringUtils.isBlank(upgradeSql)) {
            throw new RuntimeException("没有配置升级sql脚本文件地址");
        }
        File file = new File(upgradeSql);
        if (!file.exists()) {
            throw new RuntimeException("升级sql脚本文件不存在");
        }
        try {
            FileInputStream inputStream = new FileInputStream(upgradeSql);
            String[] sqlList = StringUtils.split(ResourceFileHelper.readContent(inputStream), ";");
            Arrays.stream(sqlList).filter(StringUtils::isNotBlank).forEach(sql -> {
                log.info(sql);
                jdbcTemplate.execute(sql);
            });
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        }
    }

    private <T> T getOne(JdbcTemplate jdbcTemplate, String sql, Class<T> tClass) {
        //将查询的语句封装到List集合中，集合中存储每一个员工实体
        T t = null;
        try {
            t = jdbcTemplate.queryForObject(sql, new BeanPropertyRowMapper<>(tClass));
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("查询失败：" + e.getMessage());
        }
        return t;
    }

    private <T> List<T> listData(JdbcTemplate jdbcTemplate, String sql, Class<T> tClass) {
        //将查询的语句封装到List集合中，集合中存储每一个员工实体
        List<T> list = new ArrayList<>();
        try {
            list = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(tClass));
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("查询失败：" + e.getMessage());
        }
        return list;
    }

    private int update(JdbcTemplate jdbcTemplate, String sql) {
        try {
            return jdbcTemplate.update(sql);
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("更新失败：" + e.getMessage());
        }
    }

    private <T> List<T> pageData(JdbcTemplate jdbcTemplate, String sql, Class<T> tClass) {
        if (!sql.contains("limit")) {
            sql += " limit ?,?";
        }
        //将查询的语句封装到List集合中，集合中存储每一个员工实体
        List<T> listAll = new ArrayList<>();
        try {
            List<T> list;
            int pageNumber = 0;
            do {
                int offset = pageNumber * PAGE_SIZE + PAGE_SIZE;
                list = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(tClass), pageNumber, offset);
                if (CollectionUtils.isNotEmpty(list)) {
                    listAll.addAll(list);
                    pageNumber++;
                }
            } while (CollectionUtils.isNotEmpty(list));
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("查询失败：" + e.getMessage());
        }
        return listAll;
    }

    private <T> List<T> pageData(JdbcTemplate jdbcTemplate, String sql, Class<T> tClass, int pageNumber) {
        //将查询的语句封装到List集合中，集合中存储每一个员工实体
        List<T> listAll = new ArrayList<>();
        try {
            int offset = pageNumber * PAGE_SIZE + PAGE_SIZE;
            List<T> list = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(tClass), pageNumber, offset);
            if (CollectionUtils.isNotEmpty(list)) {
                listAll.addAll(list);
            }
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("查询失败：" + e.getMessage());
        }
        return listAll;
    }

    private List<ExamTaskDetail> examTaskDetailList(JdbcTemplate jdbcTemplate) {
        String sql = "select * from exam_task_detail etd where etd.paper_confirm_attachment_ids is not null and etd.exam_task_id in (select id from exam_task)";
        return this.listData(jdbcTemplate, sql, ExamTaskDetail.class);
    }

    private ExamTask getExamTaskById(JdbcTemplate jdbcTemplate, Long examTaskId) {
        String sql = "select * from exam_task where id = " + examTaskId;
        return this.getOne(jdbcTemplate, sql, ExamTask.class);
    }

    private void updateExamDetailCourseSerialNumber(JdbcTemplate jdbcTemplate) {
        StringBuffer sql = new StringBuffer("");
        sql.append(" UPDATE exam_detail_course edc                                          ");
        sql.append("             LEFT JOIN                                                  ");
        sql.append("             exam_detail ed ON edc.exam_detail_id = ed.id               ");
        sql.append("             SET                                                        ");
        sql.append("             edc.serial_number = (SELECT                                ");
        sql.append("             etd.serial_number                                          ");
        sql.append("             FROM                                                       ");
        sql.append("             exam_task_detail etd                                       ");
        sql.append("             LEFT JOIN                                                  ");
        sql.append("             exam_task et ON etd.exam_task_id = et.id                   ");
        sql.append("             WHERE                                                      ");
        sql.append("             ed.exam_id = et.exam_id                                    ");
        sql.append("             AND edc.paper_number = et.paper_number                     ");
        sql.append("             AND edc.paper_type = etd.paper_type)                       ");
        sql.append("         WHERE                                                          ");
        sql.append("             edc.serial_number IS NULL                                  ");
        jdbcTemplate.update(sql.toString());
    }

    private void updateMarkPaperSerialNumber(JdbcTemplate jdbcTemplate) {
        String sql = "update mark_paper set serial_number = case paper_type when 'A' then 1 when 'B' then 2 when 'C' then 3 when 'D' then 4 when 'E' then 5 when 'F' then 6 when 'G' then 7 end where serial_number is null";
        jdbcTemplate.update(sql);
    }

    private void updateMarkPaperPackageSerialNumber(JdbcTemplate jdbcTemplate) {
        String sql = "update mark_paper_package set serial_number = case paper_type when 'A' then 1 when 'B' then 2 when 'C' then 3 when 'D' then 4 when 'E' then 5 when 'F' then 6 when 'G' then 7 end where serial_number is null";
        jdbcTemplate.update(sql);
    }

    private void updateMarkStudentSerialNumber(JdbcTemplate jdbcTemplate) {
        String sql = "update mark_student set serial_number = case paper_type when 'A' then 1 when 'B' then 2 when 'C' then 3 when 'D' then 4 when 'E' then 5 when 'F' then 6 when 'G' then 7 end where serial_number is null";
        jdbcTemplate.update(sql);
    }

    private void updateScanAnswerCardSerialNumber(JdbcTemplate jdbcTemplate) {
        String sql = "update scan_answer_card set serial_number = case paper_type when 'A' then 1 when 'B' then 2 when 'C' then 3 when 'D' then 4 when 'E' then 5 when 'F' then 6 when 'G' then 7 end where serial_number is null";
        jdbcTemplate.update(sql);
    }

    private List<MarkQuestion> markQuestionList(JdbcTemplate jdbcTemplate) {
        String sql = "select * from mark_question where objective = true order by exam_id, paper_number";
        return this.listData(jdbcTemplate, sql, MarkQuestion.class);
    }

    private long saveBatchExamTaskDetail(JdbcTemplate jdbcTemplate, List<ExamTaskDetail> list) {
        String sql = "insert into exam_task_detail(exam_task_id, serial_number, paper_type, paper_attachment_ids, exposed, exposed_exam_id, create_id, create_time, update_id, update_time) values (?,?,?,?,?,?,?,?,?,?)";
        int[] ints = jdbcTemplate.batchUpdate(sql, new BatchPreparedStatementSetter() {
            @Override
            public void setValues(PreparedStatement preparedStatement, int i) throws SQLException {
                final ExamTaskDetail t = list.get(i);
                // 参数赋值的时候，序号是从 1 开始的，这一点要注意
                if (t.getExamTaskId() != null) {
                    preparedStatement.setLong(1, t.getExamTaskId());
                } else {
                    preparedStatement.setNull(1, Types.BIGINT);
                }
                if (t.getSerialNumber() != null) {
                    preparedStatement.setInt(2, t.getSerialNumber());
                } else {
                    preparedStatement.setNull(2, Types.INTEGER);
                }
                if (StringUtils.isNotBlank(t.getPaperType())) {
                    preparedStatement.setString(3, t.getPaperType());
                } else {
                    preparedStatement.setNull(3, Types.VARCHAR);
                }
                if (StringUtils.isNotBlank(t.getPaperAttachmentIds())) {
                    preparedStatement.setString(4, t.getPaperAttachmentIds());
                } else {
                    preparedStatement.setNull(4, Types.VARCHAR);
                }
                if (t.getExposed() != null) {
                    preparedStatement.setBoolean(5, t.getExposed());
                } else {
                    preparedStatement.setNull(5, Types.BOOLEAN);
                }
                if (t.getExposedExamId() != null) {
                    preparedStatement.setLong(6, t.getExposedExamId());
                } else {
                    preparedStatement.setNull(6, Types.INTEGER);
                }
                if (t.getCreateId() != null) {
                    preparedStatement.setLong(7, t.getCreateId());
                } else {
                    preparedStatement.setNull(7, Types.BIGINT);
                }
                if (t.getCreateTime() != null) {
                    preparedStatement.setLong(8, t.getCreateTime());
                } else {
                    preparedStatement.setNull(8, Types.BIGINT);
                }
                if (t.getUpdateId() != null) {
                    preparedStatement.setLong(9, t.getUpdateId());
                } else {
                    preparedStatement.setNull(9, Types.BIGINT);
                }
                if (t.getUpdateTime() != null) {
                    preparedStatement.setLong(10, t.getUpdateTime());
                } else {
                    preparedStatement.setNull(10, Types.BIGINT);
                }

            }

            @Override
            public int getBatchSize() {
                return list.size();
            }
        });
        return Arrays.stream(ints).mapToLong(m -> m).sum();
    }

    private long updateBatchExamTaskDetail(JdbcTemplate jdbcTemplate, List<ExamTaskDetail> list) {
        String sql = "update exam_task_detail set serial_number = ?, paper_type = ?, paper_attachment_ids = ?, exposed = ?, exposed_exam_id = ? where id = ?";
        int[] ints = jdbcTemplate.batchUpdate(sql, new BatchPreparedStatementSetter() {
            @Override
            public void setValues(PreparedStatement preparedStatement, int i) throws SQLException {
                final ExamTaskDetail t = list.get(i);
                // 参数赋值的时候，序号是从 1 开始的，这一点要注意
                if (t.getSerialNumber() != null) {
                    preparedStatement.setInt(1, t.getSerialNumber());
                } else {
                    preparedStatement.setNull(1, Types.INTEGER);
                }
                if (StringUtils.isNotBlank(t.getPaperType())) {
                    preparedStatement.setString(2, t.getPaperType());
                } else {
                    preparedStatement.setNull(2, Types.VARCHAR);
                }
                if (StringUtils.isNotBlank(t.getPaperAttachmentIds())) {
                    preparedStatement.setString(3, t.getPaperAttachmentIds());
                } else {
                    preparedStatement.setNull(3, Types.VARCHAR);
                }
                if (t.getExposed() != null) {
                    preparedStatement.setBoolean(4, t.getExposed());
                } else {
                    preparedStatement.setNull(4, Types.BOOLEAN);
                }
                if (t.getExposedExamId() != null) {
                    preparedStatement.setLong(5, t.getExposedExamId());
                } else {
                    preparedStatement.setNull(5, Types.BIGINT);
                }
                preparedStatement.setLong(6, t.getId());
            }

            @Override
            public int getBatchSize() {
                return list.size();
            }
        });
        return Arrays.stream(ints).mapToLong(m -> m).sum();
    }

    private long saveBatchMarkQuestionAnswer(JdbcTemplate jdbcTemplate, List<MarkQuestionAnswer> list) {
        String sql = "insert into mark_question_answer(exam_id, paper_number, paper_type, main_number, sub_number, answer, objective_policy, objective_policy_score) values (?,?,?,?,?,?,?,?)";
        int[] ints = jdbcTemplate.batchUpdate(sql, new BatchPreparedStatementSetter() {
            @Override
            public void setValues(PreparedStatement preparedStatement, int i) throws SQLException {
                final MarkQuestionAnswer mas = list.get(i);
                // 参数赋值的时候，序号是从 1 开始的，这一点要注意
                preparedStatement.setLong(1, mas.getExamId());
                preparedStatement.setString(2, mas.getPaperNumber());
                preparedStatement.setString(3, mas.getPaperType());
                preparedStatement.setInt(4, mas.getMainNumber());
                preparedStatement.setInt(5, mas.getSubNumber());
                if (StringUtils.isNotBlank(mas.getAnswer())) {
                    preparedStatement.setString(6, mas.getAnswer());
                } else {
                    preparedStatement.setNull(6, Types.VARCHAR);
                }
                if (mas.getObjectivePolicy() != null) {
                    preparedStatement.setString(7, mas.getObjectivePolicy().name());
                } else {
                    preparedStatement.setNull(7, Types.VARCHAR);
                }
                if (mas.getObjectivePolicyScore() != null) {
                    preparedStatement.setDouble(8, mas.getObjectivePolicyScore());
                } else {
                    preparedStatement.setNull(8, Types.DOUBLE);
                }
            }

            @Override
            public int getBatchSize() {
                return list.size();
            }
        });
        return Arrays.stream(ints).mapToLong(m -> m).sum();
    }

    private String getPaperTypeIds(List<PaperInfoVo> paperInfoVoList, String oldPaperType) {
        PaperInfoVo paperInfoVo = paperInfoVoList.stream().filter(m -> m.getName().equals(oldPaperType)).findFirst().get();
        paperInfoVo.setName(oldPaperType);

        List<PaperInfoVo> list = new ArrayList<>();
        list.add(paperInfoVo);
        return JSON.toJSONString(list);
    }

    private Integer calcSerialNumber(String paperType) {
        String[] letter = {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L"};
        List<String> letterList = Arrays.asList(letter);
        return letterList.indexOf(paperType) + 1;
    }
}
